import {BehaviorSubject, catchError, of, Subject, Subscription} from "rxjs";
import {Injectable} from "@angular/core";
import {AuthService} from "@service/auth.service";
import {environment} from "@environment";
import ApiService from "@api/api.service";

@Injectable({
  providedIn: 'root',
})
export class TokenRefreshService {
  private tokenRefreshing$ = new BehaviorSubject<boolean>(false);
  private refreshTokenTimeout: any;
  private refreshTokenExpireMs: number = environment.retryDelay;

  constructor(private authService: AuthService) {
    this.setRefreshTokenExpireMs(AuthService.tokenExpiresIn);
  }

  private startTokenRefresh(): void {
    this.refreshTokenTimeout = setTimeout(() => {
      this.refreshToken();

      if (this.refreshTokenExpireMs > 0)
        this.startTokenRefresh();
    }, this.refreshTokenExpireMs);
  }

  private refreshToken(): void {
    if (this.tokenRefreshing$.value) {
      console.warn('Refreshing token was started...');
      return;
    }

    console.log('Start Refreshing token...');
    this.tokenRefreshing$.next(true);

    this.authService.refreshToken()
      .pipe(
        catchError(error => {
          console.warn('Error refreshing token', error);
          localStorage.removeItem(ApiService.tokenKey);
          this.refreshTokenExpireMs = -1;
          return of(undefined);
        }))
      .subscribe(data => {
        if (data) {
          console.log('Token refreshed');
          this.setRefreshTokenExpireMs(data);
        }
        this.tokenRefreshing$.next(false);
        console.log('Token refresh process completed');
      });
  }

  public getTokenRefreshing$(): Subject<boolean> {
    return this.tokenRefreshing$;
  }

  public setRefreshTokenExpireMs(expireMs: number | string | Date | null = null): void {
    let expireMsNumber: number;
    if (expireMs === null)
      expireMsNumber = -1;
    else if (expireMs instanceof Date || typeof expireMs === 'string')
      expireMsNumber = new Date(expireMs).getTime() - 1000 - Date.now();
    else
      expireMsNumber = expireMs;

    if (expireMsNumber < environment.retryDelay)
      expireMsNumber = environment.retryDelay;

    this.refreshTokenExpireMs = expireMsNumber;
    console.log('New refresh token interval:', this.refreshTokenExpireMs);

    clearTimeout(this.refreshTokenTimeout);
    this.startTokenRefresh();
  }
}
